#
# EKS Worker Nodes Resources
#  * IAM role allowing Kubernetes actions to access other AWS services
#  * EC2 Security Group to allow networking traffic
#  * Data source to fetch latest EKS worker AMI
#  * AutoScaling Launch Configuration to configure worker instances
#  * AutoScaling Group to launch worker instances
#

resource "aws_iam_role" "susecap-node" {
  name = "susecap-ec2-role"

  assume_role_policy = <<POLICY
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Effect": "Allow",
      "Principal": {
        "Service": "ec2.amazonaws.com"
      },
      "Action": "sts:AssumeRole"
    }
  ]
}
POLICY
}

resource "aws_iam_role_policy_attachment" "susecap-node-AmazonEKSWorkerNodePolicy" {
  policy_arn = "arn:aws:iam::aws:policy/AmazonEKSWorkerNodePolicy"
  role       = aws_iam_role.susecap-node.name
}

resource "aws_iam_role_policy_attachment" "susecap-node-AmazonEKS_CNI_Policy" {
  policy_arn = "arn:aws:iam::aws:policy/AmazonEKS_CNI_Policy"
  role       = aws_iam_role.susecap-node.name
}

resource "aws_iam_role_policy_attachment" "susecap-node-AmazonEC2ContainerRegistryReadOnly" {
  policy_arn = "arn:aws:iam::aws:policy/AmazonEC2ContainerRegistryReadOnly"
  role       = aws_iam_role.susecap-node.name
}

resource "aws_iam_instance_profile" "susecap-node" {
  name = "terraform-eks-susecap"
  role = aws_iam_role.susecap-node.name
}

resource "aws_security_group" "susecap-node" {
  name        = "terraform-eks-susecap-node"
  description = "Security group for all nodes in the cluster"
  vpc_id      = aws_vpc.susecap.id

  egress {
    from_port   = 0
    to_port     = 0
    protocol    = "-1"
    cidr_blocks = ["0.0.0.0/0"]
  }

  tags = map(
     "Name", "terraform-eks-susecap-node",
     "kubernetes.io/cluster/${var.cluster-name}", "owned",
    )
}

resource "aws_security_group_rule" "susecap-node-ingress-self" {
  description              = "Allow node to communicate with each other"
  from_port                = 0
  protocol                 = "-1"
  security_group_id        = aws_security_group.susecap-node.id
  source_security_group_id = aws_security_group.susecap-node.id
  to_port                  = 65535
  type                     = "ingress"
}

resource "aws_security_group_rule" "susecap-node-ingress-cluster" {
  description              = "Allow worker Kubelets and pods to receive communication from the cluster control plane"
  from_port                = 1025
  protocol                 = "tcp"
  security_group_id        = aws_security_group.susecap-node.id
  source_security_group_id = aws_security_group.susecap-cluster.id
  to_port                  = 65535
  type                     = "ingress"
}

data "aws_ami" "eks-worker" {
  filter {
    name   = "name"
    values = ["amazon-eks-node-1.14-v2019*"]
  }

  most_recent = true
  owners      = ["602401143452"] # Amazon EKS AMI Account ID
}

# EKS currently documents this required userdata for EKS worker nodes to
# properly configure Kubernetes applications on the EC2 instance.
# We utilize a Terraform local here to simplify Base64 encoding this
# information into the AutoScaling Launch Configuration.
# More information: https://docs.aws.amazon.com/eks/latest/userguide/launch-workers.html
locals {
  susecap-node-userdata = <<USERDATA
#!/bin/bash
set -o xtrace
/etc/eks/bootstrap.sh --apiserver-endpoint '${aws_eks_cluster.susecap.endpoint}' --b64-cluster-ca '${aws_eks_cluster.susecap.certificate_authority.0.data}' '${var.cluster-name}'
USERDATA
}

resource "aws_launch_configuration" "susecap" {
  associate_public_ip_address = true
  iam_instance_profile        = aws_iam_instance_profile.susecap-node.name
  image_id                    = data.aws_ami.eks-worker.id
  instance_type               = var.cluster-instance-type
  name_prefix                 = var.cluster-name
  security_groups             = [aws_security_group.susecap-node.id]
  user_data_base64            = base64encode(local.susecap-node-userdata)

  root_block_device {
    volume_type           = "gp2"
    volume_size           = 100
    delete_on_termination = true
  }

  lifecycle {
    create_before_destroy = true
  }
}

resource "aws_autoscaling_group" "susecap" {
  desired_capacity     = var.cluster-desire-size
  launch_configuration = aws_launch_configuration.susecap.id
  max_size             = var.cluster-max-size
  min_size             = var.cluster-min-size
  name                 = var.cluster-name
  vpc_zone_identifier  = ["${aws_subnet.susecap.*.id}"]

  tag {
    key                 = "Name"
    value               = var.cluster-name
    propagate_at_launch = true
  }

  tag {
    key                 = "kubernetes.io/cluster/${var.cluster-name}"
    value               = "owned"
    propagate_at_launch = true
  }
}


